/*
 * Copyright (C) 2009-2011 Dennis Schneider
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *		notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *		notice, this list of conditions and the following disclaimer in the
 *		documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <avr/wdt.h>
#include <string.h>
#include <stdio.h>
#include "config.h"
#include "avrutils.h"
#include "amish.h"
#include "amish_uart.h"
#include "amish_uip.h"

void amish_init(void) {
	amish_uart_init();
}


void amish_worker(void) {
	amish_uart_worker();
}


amish_cmd* amish_firstexec_cmd(amish_cmd_arg *ca) {
	amish_cmd *cmdP = (amish_cmd*) amish_cmds;
	amish_cmd cmd;

	while(1) {
		uint8_t cmd_len;
		/* Get the current command tuple. */
		memcpy_P(&cmd, cmdP, sizeof(amish_cmd));
		/* End of list? */
		if(!cmd.name) break;

		cmd_len = strlen_P(cmd.name);
		/* Is the command equal to the first characters of the input? */
		if(!strncmp_P(ca->input, cmd.name, cmd_len)) {
			/* Found it. */

			/* Shift forward the input pointer so that only its arguments remain */
			ca->input += cmd_len;
			break;
		}

		cmdP++;
	}

	if(!cmd.name) {
		strcpy_P(ca->output, PSTR("Unknown command."));
		ca->ret.call_again = 0;
		ca->ret.len = 16;
	} else {
		/* Initialize the command state: */
		ca->arg = cmd.arg;
		ca->state.val = 0;
		cmd.handler(ca);
		if(ca->ret.state == AMISH_RET_STATE_FAILURE) {
			strcpy_P(ca->output, PSTR("Command failed."));
			ca->ret.call_again = 0;
			ca->ret.len = 15;
		}
	}
	return cmdP;
}


void amish_reexec_cmd(amish_cmd* cmd, amish_cmd_arg* ca) {
	/*
	 * We have to supply the same fixed argument to the
	 * command and, more importantly, correct the input offset
	 * by the length of the command.
	 */
	int8_t cmd_arg = pgm_read_byte(&cmd->arg);
	const prog_char* cmd_name = (const prog_char*) pgm_read_word(&cmd->name);
	uint8_t input_off = (uint8_t) strlen_P(cmd_name);
	AMISH_HANDLER_FPOINTER(cmd_handler) =
		(AMISH_HANDLER_FPOINTER( )) pgm_read_word(&cmd->handler);

	ca->arg = cmd_arg;
	ca->input += input_off;

	cmd_handler(ca);
	if(ca->ret.state == AMISH_RET_STATE_FAILURE) {
		strcpy_P(ca->output, PSTR("Command failed."));
		ca->ret.call_again = 0;
		ca->ret.len = 15;
	}
}


/******************************* Commands *******************************/

void amish_cmd_help(amish_cmd_arg *ca) {
	amish_cmd_ret ret;
	amish_cmd *cmd = (amish_cmd*) ca->state.lo;
	const prog_char *cmd_name;

	ret.val = 0;
	if(ca->state.hi != 1) {
		ca->state.hi = 1;
		strncpy_P(ca->output, PSTR("Available commands:"), ca->len);
		ca->ret.len = 19;
		ca->ret.call_again = 1;
		cmd = (amish_cmd*) amish_cmds;
	} else { /* Already initialized */
		/* Retrieve the name of the command and print it */
		cmd_name = (const prog_char*) pgm_read_word(&cmd->name);
		strncpy_P(ca->output, cmd_name, ca->len);
		ca->ret.len = strlen_P(cmd_name);
		cmd++;

		/* Is there a command name left? */
		ca->ret.call_again = pgm_read_word(&cmd->name) != 0;
	}
	ca->state.lo = (uint16_t) cmd;
}

void amish_cmd_sys(amish_cmd_arg *ca) {
	char* out = ca->output;
	char* in = ca->input;
	uint8_t arg = ca->arg;
	switch(arg) {
	case CMD_SYS_RESET:
		reboot_device();
		break;

	case CMD_SYS_BL:
		reboot_to_bootloader();
		break;
	case CMD_SYS_INFO:
		out = stpcpy_P(out, PSTR("Version " FQ_VERSION "	 "));
		break;

	case CMD_SYS_PW:
		skip_whitespaces(in);
		if(*in) {
			strncpy(amish_uip_pw, in, AMISH_PW_LEN);
		}
		out = stpcpy_P(out, PSTR("Password: "));
		out = stpcpy(out, amish_uip_pw);
		break;
	/* case CMD_SYS_EXIT: ignore */
	}
	ca->ret.val = 0;
	ca->ret.len = out - ca->output;
}

