/*
 * Copyright (C) 2009-2010 Dennis Schneider
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include "config.h"
#include "amish.h"
#include "amish_cmd_uip.h"
#include "uip/uip.h"
#include "enc0.h"

void amish_cmd_uip(amish_cmd_arg *ca) {
	ca->ret.val = 0;
	if(ca->len < 18) {
		ca->ret.state = AMISH_RET_STATE_FAILURE;
	} else {
		char *in = ca->input;
		union {
			uip_ipaddr_t *ip;
			struct uip_eth_addr *eth;
		} addr;
		uint8_t field = (ca->arg & CMD_UIP_FIELD);
		if(field == CMD_UIP_FIELD_IP) {
			addr.ip = &uip_hostaddr;
		} else if(field == CMD_UIP_FIELD_GW) {
			addr.ip = &uip_draddr;
		} else if(field == CMD_UIP_FIELD_MASK) {
			addr.ip = &uip_netmask;
		} else if(field == CMD_UIP_FIELD_ETHER) {
			addr.eth = &uip_ethaddr;
		}
		
		skip_whitespaces(in);
		if(*in) {
			if((ca->arg & CMD_UIP_TYPE) == CMD_UIP_DQUAD) {
				if(!(enc0_sscanip(in, addr.ip))) ca->ret.state = AMISH_RET_STATE_FAILURE;
			} else if ((ca->arg & CMD_UIP_TYPE) == CMD_UIP_COLSEP) {
				if(!(enc0_sscanmac(in, addr.eth))) ca->ret.state = AMISH_RET_STATE_FAILURE;
				enc0_update_mac();
			}
		}

		if(ca->ret.state != AMISH_RET_STATE_FAILURE) {
			if((ca->arg & CMD_UIP_TYPE) == CMD_UIP_DQUAD) {
				enc0_sputip(ca->output, addr.ip);
			} else if ((ca->arg & CMD_UIP_TYPE) == CMD_UIP_COLSEP) {
				enc0_sputmac(ca->output, addr.eth);
			}
			ca->ret.len = strlen(ca->output);
		}
	}
}
