/*
 * Copyright (C) 2009-2011 Dennis Schneider
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *		notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *		notice, this list of conditions and the following disclaimer in the
 *		documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef _FREQCOUNTER_CONFIG_H
#define _FREQCOUNTER_CONFIG_H

#include <avr/io.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include <util/atomic.h>
#include <inttypes.h>

/* User variable defines: */

#define FQ_VERSION "1.7.0"

/* Enable debug messages?
 * most debug messages: level 2
 * normal debug messages: level 1
 * no debug messages: level 0
 */
#define DEBUG_LEVEL 0

/*
 * Define DEBUGGING_UART to replace the UART FIFO by a blocking loop.
 * Useful to employ printf() in ISRs.
 */
/* #define DEBUGGING_UART 1 */

/* Baud date for communication over the serial interface: */
#define UART_BAUD_RATE 9600

/* Size of the bootloader in words, to jump into the bootloader: */
#define REBOOT_BL_SIZE 512

/* Pins on port A: */
#define DCF77_PULSE_PIN A,0
#define ADC_S_METER 1
#define ADC_PHASE 2
#define PIN_LOCKED A,3
#define ADC_PULSE_SIGNAL 4
#define ADC_SUPPLY_VOLTAGE 5

/* Pins on port D: */
#define FN_BUTTON D,2 /* The lonely button on the front panel */
#define REBOOT_BL_BUTTON D,2 /* Bootloader button */
#define REBOOT_BL_ACTIVE 1 /* Bootloader is called at BL_BUTTON == low */

/* Maximal display brightness: */
#define DISPLAY_MAX_BRIGHTNESS 3 /* 3 + 6 = 12 mA */

/* Is the ICP1 used to generate timestamps for the DCF77 pulses? */
#define USE_PULSE_ICP 1

/* Delay correction of the DCF77 receiver and data processing (in us): */
#define DCF77_DELAY_COMPENSATION 13320

/* Network defaults: */
/*
 *XXX: Keep second bit in the first byte set, as it indicates a locally
 * administered address!
 */
#define NET_ETHER "\x02\x0a\xDE\x0D\xCF\x77" /* 02:0a:DE:0D:CF:77 */
#define NET_IP	 "\xC0\xA8\x7B\x2A" /* 192.168.123.042 */
#define NET_MASK "\xFF\xFF\xFF\x00" /* 255.255.255.000 */
#define NET_GW	 "\xC0\xA8\x7B\xFE" /* 192.168.123.254 */

/* Name this device at the prompt: */
#define AMISH_PROMPT "fnorm"
/* AMISH default password for network access: */
#define AMISH_UIP_DEFAULT_PW "-"

/* Configuration of the Ethernet controller ENC28J60: */
#define SPI_PRESCALER 2
#define ENC0_RESET D,4
#define ENC0_SCK	 B,7
#define ENC0_MISO	B,6
#define ENC0_MOSI	B,5
#define ENC0_CS		B,4
/*
 * Undefine ENC0_IRQ* to use only polling for RX/TX.
 * Interrupt operation eleminates unneeded SPI communication.
 */

/* ATmega644: */

#define ENC0_IRQ	 INT1_vect
#define ENC0_IRQ_PIN B,1
#define ENC0_IRQ_SETUP {SET_INPUT_WITH_PULLUP(ENC0_IRQ_PIN); \
	EICRA &= ~(1 << ISC10); \
	EICRA |= (1 << ISC11); \
	EIMSK |= (1 << INT1); \
	}

/* ATmega32: */
/*
#define ENC0_IRQ	 INT2_vect
#define ENC0_IRQ_PIN B,2
#define ENC0_IRQ_SETUP	{SET_INPUT_WITH_PULLUP(ENC0_IRQ_PIN); \
	MCUCSR &= ~(1 << ISC2); \
	GICR |= (1 << INT2); \
	}
*/

/* Maximum ethernet packet length for the Ethernet controller: */
#define UIP_MAX_ETH_FRAME_LENGTH 250

/* Time zone of the local timer: */
#define TIMER_TZ 60 /* 60 minutes is CET / GMT+1 */

/* Forward declaration, since strings.h doesn't exist. */
void bzero(void *b, size_t len);

/* Compatibility for ATmega32 & co: */
#ifndef TIMSK1
	#define TIMSK1 TIMSK
	#define TIFR1 TIFR
	#define ICIE1 TICIE1
#endif

#ifndef UCSR0A
	#define UCSR0A UCSRA
	#define UCSR0B UCSRB
	#define UCSR0C UCSRC
	#define UDRIE0 UDRIE
	#define USART0_UDRE_vect USART_UDRE_vect
	#define USART0_RX_vect USART_RXC_vect
	#define UDR0 UDR
	#define UDRE0 UDRE
	#define FE0 FE
	#define DOR0 DOR
	#define UPE0 PE
	#define TXC0 TXC
	#define TXEN0 TXEN
	#define RXEN0 RXEN
	#define RXCIE0 RXCIE
	#define UCSZ00 UCSZ0
	#define USBS0 URSEL
	#define UBRR0H UBRRH
	#define UBRR0L UBRRL
#endif

#ifndef EE_READY_vect
	#define EE_READY_vect EE_RDY_vect
#endif


#endif
