/*
 * Copyright (C) 2009-2010 Dennis Schneider
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef _DCF77_H
#define _DCF77_H

/* State of the DCF77 decoder: */
struct dcf77_info {
	uint8_t edge:1;
	uint8_t expect_dst_change:1;
	uint8_t second:6;
	uint8_t expect_leapsecond:1;
	uint8_t time_valid:1;
	uint8_t invalid_frames:2;
#define DCF77_MAX_INVALID_FRAMES 2
	uint8_t timer_in_sync:1;
	uint8_t set_time:1;
	uint8_t tick_stat_valid:1;
	uint8_t first_set:1;
};
extern struct dcf77_info dcf77_info;

/*
 * DCF77 reception delay that is compensated (in ticks):
 */
extern uint16_t dcf77_recv_delay;

extern uint32_t mean_edge_ticks; /* Sum of the ticks at the falling edge of one minute */
extern uint16_t mean_edge_counter; /* Number of values in mean_edge_ticks */

/*
 * dcf77_init:
 * Initializes the DCF 77 sub program
 */
void dcf77_init(void);

/*
 * dcf77_worker:
 * Processing of DCF77 events. Call it often. ;)
 */
void dcf77_worker(void);

/* 
 * dcf77_clear_datagrams:
 * Invalidate datagram to reflect a possible change in the transmitted data:
 * This occurs e.g. on DST, hour or day changes.
 */
void dcf77_clear_datagrams(void);

/*
 * dcf77_clear_sync:
 * Tell the DCF77 part that the local clock is no longer synchronized.
 */
void dcf77_clear_sync(void);

/*
 * dcf77_is_minute_end:
 * Service function to return a nonzero value if a minute end condition
 * was detected. Warning: The function is a bit heavy.
 */
uint8_t dcf77_is_minute_end(void);

/*
 * Convert a timer tick counter to an delay in microseconds and back: 
 */
uint16_t dcf77_ticks2delay(uint16_t ticks);
uint16_t dcf77_delay2ticks(uint16_t delay);

#endif
