/*
 * ----------------------------------------------------------------------------
 * "THE BEER-WARE LICENSE" (Revision 42):
 * <joerg@FreeBSD.ORG> wrote this file.  As long as you retain this notice you
 * can do whatever you want with this stuff. If we meet some day, and you think
 * this stuff is worth it, you can buy me a beer in return.        Joerg Wunsch
 * ----------------------------------------------------------------------------
 *
 * Modified by Dennis Schneider (2009), same license applies.
 */

#include <inttypes.h>
#include <util/twi.h>
#include "avrutils.h"
#include "twi.h"

#define TWI_TRIES 100

/*
 * Activate / deactivate TWI to save power.
 */
inline void twi_on(void) {
	TWCR = _BV(TWEN);
}

inline void twi_off(void) {
	TWCR = 0;
}

void twi_init(void) {
	  /* initialize TWI<< clock: 100 kHz clock, TWPS = 0 => prescaler = 1 */
#if defined(TWPS0)
  /* has prescaler (mega128 & newer) */
  TWSR = 0;
#endif

#if F_CPU < 3600000UL
  TWBR = 10;      /* smallest TWBR value, see note [5] */
#else
  TWBR = (F_CPU / 100000UL - 16) / 2;
#endif
}

uint8_t twi_start_(uint8_t addr) {
	twi_on();
	uint8_t status, tries = TWI_TRIES;
	
	/* Send the start condition, repeatedly if neccessary */
twi_start_begin:
	if(!--tries) return 0;
	status = twi_tx_(_BV(TWSTA));
	if(status == TW_MT_ARB_LOST) goto twi_start_begin; /* rearbitrate */
	if(status != TW_START && status != TW_REP_START) return 0;

	/* The start condition is now sent. Proceed with the address and data dir. */

	TWDR = addr;
	status = twi_tx();
	switch(status) {
	case TW_MT_SLA_ACK:
		break;
	
	case TW_MT_SLA_NACK: /* Device busy? */
	case TW_MT_ARB_LOST: /* rearbitrate */
		goto twi_start_begin;
		break;
	
	default:
		goto twi_start_stop;
	}

	return 1;

twi_start_stop:
	twi_stop();
	return 0;
}

uint8_t twi_tx_(uint8_t mode) {
	TWCR = _BV(TWINT) | _BV(TWEN) | mode;
	while ((TWCR & _BV(TWINT)) == 0) ; /* wait for transmission */
	return TW_STATUS;
}

void twi_stop(void) {
	TWCR = _BV(TWINT) | _BV(TWSTO) | _BV(TWEN);
	while(TWCR & _BV(TWSTO)); /* Loop until the stop is tramsmitted. */
	twi_off();
}
