/*
 * Copyright (C) 2009-2010 Dennis Schneider
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef _DS_TWI_H
#define _DS_TWI_H

#include <inttypes.h>
#include <util/twi.h>

/*
 * twi_init:
 * Initialize the TWI (master mode)
 * Prequisite: SDA/SCL set to input!
 */
void twi_init(void);

/*
 * twi_start:
 * Transmit a start condition and if it succeeds,
 * send the address thereafter.
 * addr: The TWI target address, lowest bit should be 0.
 * dir: TW_READ or TW_WRITE
 * Return value: success: 1, failure: 0
 * You may not transmit a stop condition if twi_start returns a failure!
 * This is done by this function.
 */
#define twi_start(addr, dir) \
	twi_start_((addr) | (dir) )
uint8_t twi_start_(uint8_t addr);

/*
 * twi_tx / twi_tx_ack:
 * Start a TWI transmission without or with ACK at the end.
 * Return value: success: 1, failure: 0
 */
#define twi_tx()     twi_tx_(0)
#define twi_tx_ack() twi_tx_(_BV(TWEA))
uint8_t twi_tx_(uint8_t mode);

/*
 * twi_stop:
 * Send a stop condition.
 */
void twi_stop(void);

#endif
